#uses "CtrlFwHttp"
#uses "fwWebMonitor/fwWebMonitor.ctl"
#uses "fwWebMonitor/fwWebMonitorConsts.ctl"

uint siteCheckInterval = 0;

main()
{
  mapping queries;
  dpConnect("fwWebMonitorAgentIntervalChangeCB", FWWM_SYSTEM_DP_NAME+".siteCheckInterval");
  
  dpGet(FWWM_SYSTEM_DP_NAME+".siteCheckInterval", siteCheckInterval);
  if (siteCheckInterval == 0) {
    siteCheckInterval = FWWM_DEFAULT_SITE_CHECK_INTERVAL;
  }
  
  DebugTN("fwWebMonitor: Site check interval is ", siteCheckInterval);
  
  while(true) {
    mappingClear(queries);
    dyn_string siteEntries = fwWebMonitor_getSiteEntryNames();

    for (int i = 1; i <= dynlen(siteEntries); i++) {
      string siteEntryName = siteEntries[i];
      string queryId, error;
      string name, url;
      int expectedStatusCode;
      uint redirectionLimit;
      bool followRedirection, enabled, saveToFile = false;
      
      dpGet(siteEntryName+".url", url,
            siteEntryName+".enabled", enabled,
            siteEntryName+".expectedStatusCode", expectedStatusCode,
            siteEntryName+".followRedirection", followRedirection,
            siteEntryName+".redirectionLimit", redirectionLimit);
      if (enabled) {
        int rc = fwHttpGet(url, saveToFile, followRedirection, redirectionLimit, queryId, error);
        //DebugTN("Url '" + url + "' was requested, query id is " + queryId, "return code is " + rc, "error is " + error);
        if (rc == 0) {
          queries[queryId] = makeDynAnytype(siteEntryName, expectedStatusCode);
        } else {
            dpSet(siteEntryName+".lastStatus", rc,
                  siteEntryName+".statusText", error,
                  siteEntryName+".lastCheck", getCurrentTime(),
                  siteEntryName+".stateRaw", FWWM_MONITORED_ENTRY_STATE_ERROR);
        }
      }
    }
    
    //this loop could stop checking other websites if it stucks on a bad request.
    //however, it should be handled internally by Qt and timeout if there is no income data
    //but what about the situation when the download started but the connection was lost.
    //does the timeout handles this too? check it by lighttp with limited speed and kill the service during data transport
    //--> simulate various situations
    for (int i = 1; i <= mappinglen(queries); ) {
      string queryId = mappingGetKey(queries, i);
      
      int finished = fwHttpIsFinished(queryId);
      if (finished == 1) {
        dyn_anytype querySetting = queries[queryId];
        string siteEntryName = querySetting[1];
        int expectedStatusCode = querySetting[2];

        int statusCode = 0;
        string statusPhrase, content, redirectedLocation;
        int rc = fwHttpReadReply(queryId, content, statusCode, statusPhrase, redirectedLocation);

        //DebugTN("entry", siteEntryName, "return code from read reply function", rc, "status code", statusCode, "status phrase", statusPhrase, "redirection location", redirectedLocation);
        mappingRemove(queries, queryId);
        
        int state = (statusCode == expectedStatusCode ? FWWM_MONITORED_ENTRY_STATE_OK : FWWM_MONITORED_ENTRY_STATE_ERROR);
        dpSet(siteEntryName+".lastStatus", statusCode,
              siteEntryName+".statusText", statusPhrase,
              siteEntryName+".lastCheck", getCurrentTime(),
              siteEntryName+".stateRaw", state);

        if (i > mappinglen(queries)) {
          //this can happen when the 'last' entry (last in processing, not necessarily last existing) in mapping is processed and deleted but there could still be some 
          i = 1;
        }
      } else {
        i = ((i+1)>mappinglen(queries)) ? 1 : i+1;
      }
    }

    delay(siteCheckInterval);
  }
}


void fwWebMonitorAgentIntervalChangeCB(string dpe, uint val)
{
  if (val < 1) {
    siteCheckInterval = val;
  }
}  

